/**
  ******************************************************************************
  * @file           : adc.c
  * @brief          : ADC driver.
  * @author         : sbl1987@mail.ru
  ******************************************************************************
*/
/* Includes ------------------------------------------------------------------*/
#include "adc.h"

/* Defines -------------------------------------------------------------------*/
#define ADC_NONE_CH                    255


extern ADC_HandleTypeDef hadc1;
/* Private variables ---------------------------------------------------------*/
static uint16_t adc_data_read[ADC_MAX_CHANNAL];
static uint8_t adc_channal_t;

/* Private function prototypes -----------------------------------------------*/
extern void Error_Handler(void);

void adc_init(void)
{
  for(uint8_t i=0; i<ADC_MAX_CHANNAL; i++) adc_data_read[i] = 0;
  adc_channal_t = ADC_NONE_CH;
}

void adc_start(uint8_t adc_ch)
{
  ADC_ChannelConfTypeDef sConfig;
  
  if(adc_ch >= ADC_MAX_CHANNAL) return;
  
  if(HAL_ADC_Stop_IT(&hadc1) != HAL_OK)
  {
    Error_Handler();
  }
  
  adc_channal_t = adc_ch;
  
  if(adc_channal_t == 0) sConfig.Channel = ADC_CHANNEL_0;
  else sConfig.Channel = ADC_CHANNEL_1;
  sConfig.Rank = 1;
  sConfig.SamplingTime = ADC_SAMPLETIME_1CYCLE_5;
  if(HAL_ADC_ConfigChannel(&hadc1, &sConfig) != HAL_OK)
  {
    Error_Handler();
  }
  
  if(HAL_ADC_Start_IT(&hadc1) != HAL_OK)
  {
    Error_Handler();
  }
}

uint16_t adc_get(uint8_t adc_ch)
{
  if(adc_ch >= ADC_MAX_CHANNAL) return 0;
  
  return adc_data_read[adc_ch];
}

void HAL_ADC_ConvCpltCallback(ADC_HandleTypeDef* hadc)
{
  if(hadc->Instance == ADC1 && adc_channal_t != ADC_NONE_CH)
  {
    adc_data_read[adc_channal_t] = (uint16_t)HAL_ADC_GetValue(hadc);
  }else HAL_ADC_GetValue(hadc);
}
