/**
  ******************************************************************************
  * @file           : com.c
  * @brief          : comands parser
  * @author         : sbl1987@mail.ru
  ******************************************************************************
*/
/* Includes ------------------------------------------------------------------*/
#include "com.h"
#include "usbd_cdc_if.h"
#include "led.h"
#include "pin_out_in.h"
#include "adc.h"
#include "spi.h"

/* Defines -------------------------------------------------------------------*/
#define COM_TEXT                       "COM"
#define COM_SIM_START                  '['
#define COM_SIM_END                    ']'
#define COM_CHAR_END()                 buf_com[com_len-2]
#define COM_DEVIDER_PARAMS             '.'
#define COM_ENTER                      '\r'

/*  */
#define COM_INFO                       "?"                                      /*                    */
#define COM_LED_SET_PWM                "LED_SET"                                /*    PC13             */
#define COM_LED_GET_PWM                "LED_GET"                                /*    PC13                */

#define COM_PIN_SET                    "PIN_SET"                                /*   out                            */
#define COM_PIN_GET                    "PIN_GET"                                /*   out  in                          */

#define COM_PIN_ADC_SET                "ADC_SET"                                /*    ADC                        */
#define COM_PIN_ADC_GET                "ADC_GET"                                /*   ADC                           */

#define COM_SPI_CONFIG                 "SPI_CONF"                               /*  SPI                             */
#define COM_SPI_CONFIG_MSB             "MSB"
#define COM_SPI_CONFIG_LSB             "LSB"

#define COM_SPI_WRITE_DATA             "SPI_WR"                                 /*    SPI                        */
#define COM_SPI_READ_DATA              "SPI_RD"                                 /*    SPI                          */
#define COM_SPI_WRITE_READ_DATA        "SPI_TR"                                 /*       SPI */

/*     */
#define COM_OK                         "\n\rOK\n\r"
#define COM_NOT_FOUND                  "\n\rCOMMAND NOT FOUND !\n\r"
#define COM_INVALID                    "\n\rCOMMAND INVALID !\n\r"
#define COM_PARAM_INVALID              "\n\rPARAMETER INVALID!\n\r"

#define COM_BUF_MAX                    64

/* Private variables ---------------------------------------------------------*/
static const uint8_t info[] = 
"\n\rCOM[LED_SET.0-255]\n\rCOM[LED_GET]\n\rCOM[PIN_SET.0-3.0-1]\n\rCOM[PIN_GET.0-5]\n\rCOM[ADC_SET.0-1]\n\rCOM[ADC_GET.0-1]\n\rCOM[SPI_CONF.2,4,8,16,32,64.MSB,LSB.L,H.1,2]\n\rCOM[SPI_WR.1-32.DATA]\n\rCOM[SPI_RD.1-32]\n\rCOM[SPI_TR.1-20.DATA]\n\r";

static uint8_t buf_read_data[COM_BUF_MAX];
static uint16_t buf_read_len;

static uint8_t buf_com[COM_BUF_MAX];
static uint8_t com_len;
static uint8_t com_fl;

static uint8_t buf_write_data[COM_BUF_MAX];
static uint16_t buf_write_len;

/* Private function prototypes -----------------------------------------------*/
static uint8_t com_read_to_buf_com(void);
static void com_parser_commands(void);

static void com_transmit_info(void);

static void com_false(void);
static void com_true(void);
static void com_get_true(uint8_t *data, uint8_t len);

static uint8_t str_to_dec(uint8_t *data, uint8_t len, uint8_t *dec);
static uint8_t dec_to_str(uint8_t *data, uint16_t ldec);

void com_init(void)
{
  com_len = 0;
  com_fl = 0;
}

void com_update(void)
{
  /*    */
  if(CDC_Receive_FS_data(buf_read_data, &buf_read_len)){
    com_fl = com_read_to_buf_com();
  }
  
  /*    */
  if(com_fl == 1)
  {
    if(com_len > 6 && COM_CHAR_END() == COM_SIM_END) com_parser_commands();    
    else CDC_Transmit_FS(COM_INVALID, strlen(COM_INVALID));
    
    com_fl = 0;
    com_len = 0;
  }
}

/* Private function prototypes -----------------------------------------------*/
static uint8_t com_read_to_buf_com(void)
{
  uint8_t *adr_s = memchr(buf_read_data, COM_ENTER, buf_read_len);
  
  if(adr_s != NULL)
  {
    for(uint8_t i=0; i<COM_BUF_MAX; i++){
      buf_com[com_len] = buf_read_data[i];
      if(com_len < (COM_BUF_MAX-1)) com_len ++;
      if(adr_s == &buf_read_data[i]) break;
    }
    
    buf_com[com_len] = '\0';
    return 1;
  }else{
    if((com_len + buf_read_len) < COM_BUF_MAX){
      memcpy(&buf_com[com_len], buf_read_data, buf_read_len);
      com_len += buf_read_len;
    }
  }
  
  return 0;
}

static void com_parser_commands(void)
{
  if(strncmp((char const*)buf_com, COM_TEXT, strlen(COM_TEXT)) == 0) // "COM"
  {
    char *adr_str = strchr ((char const*)buf_com, COM_SIM_START);   // '['
    if(adr_str != NULL){
      adr_str ++;
      
      /*  */
      if(strncmp((char const*)adr_str, COM_INFO, strlen(COM_INFO)) == 0)
      {
        if(com_len == 7) com_transmit_info();
        else com_false();
        return;
      }
      
      /*    */
      else if(strncmp((char const*)adr_str, COM_LED_SET_PWM, strlen(COM_LED_SET_PWM)) == 0)
      {
        if(com_len > 14 && com_len < 18){
          char *adr_prm = strchr((char const*)adr_str, COM_DEVIDER_PARAMS); // '.'
          
          if(adr_prm != NULL){
            uint8_t dec;
            adr_prm ++;
            if(str_to_dec((uint8_t*)adr_prm, (com_len - 14), &dec) == 1){
              led_set(dec);
              com_true();
              return;
            } 
          }
        }
        com_false();
        return;
      }
      
      /*    */
      else if(strncmp((char const*)adr_str, COM_LED_GET_PWM, strlen(COM_LED_GET_PWM)) == 0)
      {
        if(com_len == 13){
          uint8_t led_data[5] = ".";
          uint8_t dec = led_get();
          uint8_t len;
          
          len = dec_to_str(&led_data[1], dec);
          len ++;
          com_get_true(led_data, len);
          return;
        }
        com_false();
        return;
      }
      
      /*   */
      else if(strncmp((char const*)adr_str, COM_PIN_SET, strlen(COM_PIN_SET)) == 0)
      {
        if(com_len == 17){
          uint8_t pin;
          uint8_t state;
          char *adr_prm = strchr((char const*)adr_str, COM_DEVIDER_PARAMS);     // '.'
          
          if(adr_prm != NULL){
            adr_prm ++;
            
            if(str_to_dec((uint8_t*)adr_prm, 1, &pin) == 1){                    //   
              adr_prm = strchr((char const*)adr_prm, COM_DEVIDER_PARAMS);       // '.'
              if(adr_prm != NULL){
                adr_prm ++;
                if(str_to_dec((uint8_t*)adr_prm, 1, &state) == 1){              //   
                  if(pin < 4 && state <= PIN_OUT_ACTIV){                        //   
                    pin_out_set(pin, state);
                    com_true();
                    return;
                  }
                }
              }
            }
          }
        }
        com_false();
        return;
      }
      
      /*    */
      else if(strncmp((char const*)adr_str, COM_PIN_GET, strlen(COM_PIN_GET)) == 0)
      {
        if(com_len == 15){
          uint8_t pin;
          char *adr_prm = strchr((char const*)adr_str, COM_DEVIDER_PARAMS);     // '.'
          
          if(adr_prm != NULL){
            adr_prm ++;
            
            if(str_to_dec((uint8_t*)adr_prm, 1, &pin) == 1){                    //  
              if(pin < 6){
                uint8_t pin_data[3] = ".";
                uint8_t dec = pin_out_in_get(pin);

                dec_to_str(&pin_data[1], dec);
                com_get_true(pin_data, 2);
                return;
              }
            }
          }
        }
        com_false();
        return;
      }
      
      /*   ADC    */
      else if(strncmp((char const*)adr_str, COM_PIN_ADC_SET, strlen(COM_PIN_ADC_SET)) == 0)
      {
        if(com_len == 15){
          uint8_t ch;
          char *adr_prm = strchr((char const*)adr_str, COM_DEVIDER_PARAMS);     // '.'
          
          if(adr_prm != NULL){
            adr_prm ++;
            
            if(str_to_dec((uint8_t*)adr_prm, 1, &ch) == 1){                     //  
              if(ch < 2){                                                       //   
                adc_start(ch);
                com_true();
                return;
              }
            }
          }
        }
        com_false();
        return;
      }
      
      /*   ADC */
      else if(strncmp((char const*)adr_str, COM_PIN_ADC_GET, strlen(COM_PIN_ADC_GET)) == 0)
      {
        if(com_len == 15){
          uint8_t ch;
          char *adr_prm = strchr((char const*)adr_str, COM_DEVIDER_PARAMS);     // '.'
          
          if(adr_prm != NULL){
            adr_prm ++;
            
            if(str_to_dec((uint8_t*)adr_prm, 1, &ch) == 1){                     //  
              if(ch < 2){
                uint8_t adc_data[6] = ".";
                uint16_t adc = adc_get(ch);
                uint8_t len;

                len = dec_to_str(&adc_data[1], adc);
                len ++;
                com_get_true(adc_data, len);
                return;
              }
            }
          }
        }
        com_false();
        return;
      }
      
      /*  SPI */
      else if(strncmp((char const*)adr_str, COM_SPI_CONFIG, strlen(COM_SPI_CONFIG)) == 0)
      {
        if(com_len == 24 || com_len == 25){
          spi_str params;
          char *adr_prm = adr_str;
          uint8_t i;
          
          for(i=0; i<4; i++)
          {
            adr_prm = strchr((char const*)adr_prm, COM_DEVIDER_PARAMS);         // '.'
            
            if(adr_prm == NULL){                                                //    
              i = 0xFF;
              break;
            }
            
            adr_prm ++;
            
            /*   */
            switch(i)
            {
            /*  2,4,8,16,32,64 */
            case 0:
              { 
                uint8_t presc;  
                if(str_to_dec((uint8_t*)adr_prm, (com_len - 23), &presc) != 1){
                  i = 0xFF;
                  break;
                }
                
                if(presc != 2 && presc != 4 && presc != 8 && presc != 16 && presc != 32 && presc != 64){
                  i = 0xFF;
                  break;
                }
                params.prescal = presc;
                break;
              }
            /*   MSB,LSB */
            case 1:
              { 
                if(strncmp((char const*)adr_prm, COM_SPI_CONFIG_MSB, strlen(COM_SPI_CONFIG_MSB)) == 0) params.F_BIT = MSB_BIT;
                else if(strncmp((char const*)adr_prm, COM_SPI_CONFIG_LSB, strlen(COM_SPI_CONFIG_LSB)) == 0) params.F_BIT = LSB_BIT;
                else i = 0xFF;
                break;
              }
              
            /*   CLK L,H */
            case 2:
              { 
                if(*adr_prm == 'L') params.CPOL = CLOW;
                else if(*adr_prm == 'H') params.CPOL = CHIGHT;
                else i = 0xFF;
                break;
              }
            
            /*   1,2 */
            case 3:
              { 
                if(*adr_prm == '1') params.CPHA = EDGE_1;
                else if(*adr_prm == '2') params.CPHA = EDGE_2;
                else i = 0xFF;
                break;
              }
            }
            
            if(i == 0xFF) break;
          }
          
          if(i != 0xFF){
            spi_init(&params);
            com_true();
            return;
          }
        }
        com_false();
        return;
      }

      /*    SPI */
      else if(strncmp((char const*)adr_str, COM_SPI_WRITE_DATA, strlen(COM_SPI_WRITE_DATA)) == 0)
      {
        if(com_len > 15 && com_len < 49){
          uint8_t len;
          uint8_t sim;
          
          char *adr_prm = strchr((char const*)adr_str, COM_DEVIDER_PARAMS);     // '.'
          
          if(adr_prm != NULL){
            adr_prm ++;
            
            if(*(adr_prm + 1) == COM_DEVIDER_PARAMS) sim = 1; else sim = 2;
            
            if(str_to_dec((uint8_t*)adr_prm, sim, &len) == 1){                  //  
              if(len > 0 && len < 33){
                if(len == (com_len - (14+sim))){
                  adr_prm = strchr((char const*)adr_prm, COM_DEVIDER_PARAMS);   // '.'
                  
                  if(adr_prm != NULL){
                    adr_prm ++;
                    spi_write((uint8_t*)adr_prm, len);
                    com_true();
                    return;
                  }
                }
              }
            }
          }
        }
        com_false();
        return;
      }
      
      /*    SPI */
      else if(strncmp((char const*)adr_str, COM_SPI_READ_DATA, strlen(COM_SPI_READ_DATA)) == 0)
      {
        if(com_len == 14 || com_len == 15){
          uint8_t len;
          uint8_t sim;
          
          char *adr_prm = strchr((char const*)adr_str, COM_DEVIDER_PARAMS);     // '.'
          
          if(adr_prm != NULL){
            adr_prm ++;
            
            if(com_len == 14) sim = 1; else sim = 2;
            
            if(str_to_dec((uint8_t*)adr_prm, sim, &len) == 1){                  //  
              if(len > 0 && len < 33){
                uint8_t data[34] = ".";
                spi_read(&data[1], len);
                len ++;
                com_get_true(data, len);
                return;
              }
            }
          }
        }
        com_false();
        return;
      }

      /*     SPI */
      else if(strncmp((char const*)adr_str, COM_SPI_WRITE_READ_DATA, strlen(COM_SPI_WRITE_READ_DATA)) == 0)
      {
        if(com_len > 15 && com_len < 37){
          uint8_t len;
          uint8_t sim;
          
          char *adr_prm = strchr((char const*)adr_str, COM_DEVIDER_PARAMS);     // '.'
          
          if(adr_prm != NULL){
            adr_prm ++;
            
            if(*(adr_prm + 1) == COM_DEVIDER_PARAMS) sim = 1; else sim = 2;
            
            if(str_to_dec((uint8_t*)adr_prm, sim, &len) == 1){                  //  
              if(len > 0 && len < 21){
                if(len == (com_len - (14+sim))){
                  adr_prm = strchr((char const*)adr_prm, COM_DEVIDER_PARAMS);   // '.'
                  
                  if(adr_prm != NULL){
                    uint8_t data[22] = ".";
                    adr_prm ++;
                    
                    spi_write_read((uint8_t*)adr_prm, &data[1], len);
                    
                    len ++;
                    com_get_true(data, len);
                    return;
                  }
                }
              }
            }
          }
        }
        com_false();
        return;
      }
      
      /*    */
      else CDC_Transmit_FS(COM_NOT_FOUND, strlen(COM_NOT_FOUND));
    }
  }
  
  CDC_Transmit_FS(COM_INVALID, strlen(COM_INVALID));
}

static void com_transmit_info(void)
{
  CDC_Transmit_FS_const(info, strlen((const char*)info));
}

static void com_false(void)
{
  CDC_Transmit_FS(COM_PARAM_INVALID, strlen(COM_PARAM_INVALID));
}

static void com_true(void)
{
  strcpy((char*)buf_write_data, COM_OK);
  strncat((char*)buf_write_data, (char*)buf_com, com_len);
  
  buf_write_len = strlen(COM_OK) + com_len;

  CDC_Transmit_FS(buf_write_data, buf_write_len);
}

static void com_get_true(uint8_t *data, uint8_t len)
{
  uint8_t len_t_str;
  
  strcpy((char*)buf_write_data, COM_OK);
  strncat((char*)buf_write_data, (char*)buf_com, com_len-2);
  
  len_t_str = strlen((char const*)buf_write_data);
  memcpy(&buf_write_data[len_t_str], data, len);
  len_t_str += len;
  memcpy(&buf_write_data[len_t_str], &COM_CHAR_END(), 2);
  
  buf_write_len = strlen(COM_OK) + com_len + len;

  CDC_Transmit_FS(buf_write_data, buf_write_len);
}

static uint8_t str_to_dec(uint8_t *data, uint8_t len, uint8_t *dec)
{
  uint16_t dec_16 = 0;
  
  if(len > 3) return 0;
  
  for(uint8_t i=0; i<len; i++){
    dec_16 *= 10;
    if(data[i] < '0' || data[i] > '9') return 0;
    dec_16 += (data[i] - '0');
  }
  
  if(dec_16 > 0xFF) return 0;
  *dec = (uint8_t)dec_16;
  
  return 1;
}

static uint8_t dec_to_str(uint8_t *data, uint16_t ldec)
{
  uint8_t len = 0;

  if(ldec > 9999){
    *data = ldec/10000 + '0';
    data ++;
    len ++;
  }
  if(ldec > 999){
    *data = (ldec%10000)/1000 + '0';
    data ++;
    len ++;
  }
  if(ldec > 99){
    *data = (ldec%1000)/100 + '0';
    data ++;
    len ++;
  }
  if(ldec > 9){
    *data = (ldec%100)/10 + '0';
    data ++;
    len ++;
  }
  *data = ldec%10 + '0';
  len ++;
  return len;
}
