/**
  ******************************************************************************
  * @file           : spi.c
  * @brief          : SPI driver.
  * @author         : sbl1987@mail.ru
  ******************************************************************************
*/
/* Includes ------------------------------------------------------------------*/
#include "spi.h"

/* Defines -------------------------------------------------------------------*/

extern SPI_HandleTypeDef hspi2;
/* Private variables ---------------------------------------------------------*/


/* Private function prototypes -----------------------------------------------*/
extern void Error_Handler(void);

void spi_init(spi_str *spi_s)
{
  if(spi_s->prescal != 2 && spi_s->prescal != 4 && spi_s->prescal != 8  && spi_s->prescal != 16 && spi_s->prescal != 32 && spi_s->prescal != 64) return;
  if(spi_s->F_BIT > LSB_BIT) return;
  if(spi_s->CPHA > EDGE_2) return;
  if(spi_s->CPOL > CHIGHT) return;
  
  if (HAL_SPI_DeInit(&hspi2) != HAL_OK)
  {
    Error_Handler();
  }
  
  hspi2.Instance = SPI2;
  hspi2.Init.Mode = SPI_MODE_MASTER;
  hspi2.Init.Direction = SPI_DIRECTION_2LINES;
  hspi2.Init.DataSize = SPI_DATASIZE_8BIT;
  
  if(spi_s->CPOL == CLOW) hspi2.Init.CLKPolarity = SPI_POLARITY_LOW;
  else hspi2.Init.CLKPolarity = SPI_POLARITY_HIGH;
  
  if(spi_s->CPHA == EDGE_1) hspi2.Init.CLKPhase = SPI_PHASE_1EDGE;
  else hspi2.Init.CLKPhase = SPI_PHASE_2EDGE;
  
  hspi2.Init.NSS = SPI_NSS_SOFT;

  switch(spi_s->prescal)
  {
    case 2:  hspi2.Init.BaudRatePrescaler = SPI_BAUDRATEPRESCALER_2;  break;
    case 4:  hspi2.Init.BaudRatePrescaler = SPI_BAUDRATEPRESCALER_4;  break;
    case 8:  hspi2.Init.BaudRatePrescaler = SPI_BAUDRATEPRESCALER_8;  break;
    case 16: hspi2.Init.BaudRatePrescaler = SPI_BAUDRATEPRESCALER_16; break;
    case 32: hspi2.Init.BaudRatePrescaler = SPI_BAUDRATEPRESCALER_32; break;
    case 64: hspi2.Init.BaudRatePrescaler = SPI_BAUDRATEPRESCALER_64; break;
    default: hspi2.Init.BaudRatePrescaler = SPI_BAUDRATEPRESCALER_64; break;
  }
  
  if(spi_s->F_BIT == MSB_BIT) hspi2.Init.FirstBit = SPI_FIRSTBIT_MSB;
  else hspi2.Init.FirstBit = SPI_FIRSTBIT_LSB;
  
  hspi2.Init.TIMode = SPI_TIMODE_DISABLE;
  hspi2.Init.CRCCalculation = SPI_CRCCALCULATION_DISABLE;
  hspi2.Init.CRCPolynomial = 10;
  
  if (HAL_SPI_Init(&hspi2) != HAL_OK)
  {
    Error_Handler();
  }
}

void spi_write(uint8_t *data, uint8_t len)
{
  HAL_SPI_Transmit(&hspi2, data, (uint16_t)len, SPI_TIMEOUT);
  
}

void spi_read(uint8_t *data, uint8_t len)
{
  HAL_SPI_Receive(&hspi2, data, (uint16_t)len, SPI_TIMEOUT);
}

void spi_write_read(uint8_t *data_w, uint8_t *data_r, uint8_t len)
{
  HAL_SPI_TransmitReceive(&hspi2, data_w, data_r, (uint16_t)len, SPI_TIMEOUT);
}
